@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -243,15 +241,16 @@ import( IOBUF inp, const char* fname,struct stats_s *stats,
 	unsigned char **fpr,size_t *fpr_len,unsigned int options )
 {
     PACKET *pending_pkt = NULL;
-    KBNODE keyblock;
+    KBNODE keyblock = NULL;
     int rc = 0;
 
     getkey_disable_caches();
 
     if( !opt.no_armor ) { /* armored reading is not disabled */
-	armor_filter_context_t *afx = xmalloc_clear( sizeof *afx );
+	armor_filter_context_t *afx = new_armor_context ();
 	afx->only_keyblocks = 1;
-	iobuf_push_filter2( inp, armor_filter, afx, 1 );
+	push_armor_filter (afx, inp);
+        release_armor_context (afx);
     }
 
     while( !(rc = read_block( inp, &pending_pkt, &keyblock) )) {
@@ -563,8 +562,11 @@ print_import_check (PKT_public_key * pk, PKT_user_id * id)
 static void
 check_prefs_warning(PKT_public_key *pk)
 {
-  log_info(_("WARNING: key %s contains preferences for unavailable\n"
-             "algorithms on these user IDs:\n"), keystr_from_pk(pk));
+  log_info(_("WARNING: key %s contains preferences for unavailable\n"),
+            keystr_from_pk(pk));
+  /* TRANSLATORS: This string is belongs to the previous one.  They are
+     only split up to allow printing of a common prefix. */
+  log_info(_("         algorithms on these user IDs:\n"));
 }
 
 static void
@@ -1640,90 +1642,102 @@ delete_inv_parts( const char *fname, KBNODE keyblock,
  * It may happen that the imported keyblock has duplicated user IDs.
  * We check this here and collapse those user IDs together with their
  * sigs into one.
- * Returns: True if the keyblock hash changed.
+ * Returns: True if the keyblock has changed.
  */
 int
 collapse_uids( KBNODE *keyblock )
 {
-    KBNODE n, n2;
-    int in_uid;
-    int any=0;
+  KBNODE uid1;
+  int any=0;
 
-  restart:
-    for( n = *keyblock; n; n = n->next ) {
-	if( n->pkt->pkttype != PKT_USER_ID )
+  for(uid1=*keyblock;uid1;uid1=uid1->next)
+    {
+      KBNODE uid2;
+
+      if(uid1->pkt->pkttype!=PKT_USER_ID)
+	continue;
+
+      for(uid2=uid1->next;uid2;uid2=uid2->next)
+	{
+	  if(uid2->pkt->pkttype!=PKT_USER_ID)
 	    continue;
-	for( n2 = n->next; n2; n2 = n2->next ) {
-	    if( n2->pkt->pkttype == PKT_USER_ID
-		&& !cmp_user_ids( n->pkt->pkt.user_id,
-				  n2->pkt->pkt.user_id ) ) {
-		/* found a duplicate */
-		any = 1;
-		if( !n2->next
-		    || n2->next->pkt->pkttype == PKT_USER_ID
-		    || n2->next->pkt->pkttype == PKT_PUBLIC_SUBKEY
-		    || n2->next->pkt->pkttype == PKT_SECRET_SUBKEY  ) {
-		    /* no more signatures: delete the user ID
-		     * and start over */
-		    remove_kbnode( keyblock, n2 );
+
+	  if(cmp_user_ids(uid1->pkt->pkt.user_id,
+			  uid2->pkt->pkt.user_id)==0)
+	    {
+	      /* We have a duplicated uid */
+	      KBNODE sig1,last;
+
+	      any=1;
+
+	      /* Now take uid2's signatures, and attach them to
+		 uid1 */
+	      for(last=uid2;last->next;last=last->next)
+		{
+		  if(last->next->pkt->pkttype==PKT_USER_ID
+		     || last->next->pkt->pkttype==PKT_PUBLIC_SUBKEY
+		     || last->next->pkt->pkttype==PKT_SECRET_SUBKEY)
+		    break;
 		}
-		else {
-		    /* The simple approach: Move one signature and
-		     * then start over to delete the next one :-( */
-		    move_kbnode( keyblock, n2->next, n->next );
+
+	      /* Snip out uid2 */
+	      (find_prev_kbnode(*keyblock,uid2,0))->next=last->next;
+
+	      /* Now put uid2 in place as part of uid1 */
+	      last->next=uid1->next;
+	      uid1->next=uid2;
+	      remove_kbnode(keyblock,uid2);
+
+	      /* Now dedupe uid1 */
+	      for(sig1=uid1->next;sig1;sig1=sig1->next)
+		{
+		  KBNODE sig2;
+
+		  if(sig1->pkt->pkttype==PKT_USER_ID
+		     || sig1->pkt->pkttype==PKT_PUBLIC_SUBKEY
+		     || sig1->pkt->pkttype==PKT_SECRET_SUBKEY)
+		    break;
+
+		  if(sig1->pkt->pkttype!=PKT_SIGNATURE)
+		    continue;
+
+		  for(sig2=sig1->next,last=sig1;sig2;last=sig2,sig2=sig2->next)
+		    {
+		      if(sig2->pkt->pkttype==PKT_USER_ID
+			 || sig2->pkt->pkttype==PKT_PUBLIC_SUBKEY
+			 || sig2->pkt->pkttype==PKT_SECRET_SUBKEY)
+			break;
+
+		      if(sig2->pkt->pkttype!=PKT_SIGNATURE)
+			continue;
+
+		      if(cmp_signatures(sig1->pkt->pkt.signature,
+					sig2->pkt->pkt.signature)==0)
+			{
+			  /* We have a match, so delete the second
+			     signature */
+			  remove_kbnode(&uid1,sig2);
+			  sig2=last;
+			}
+		    }
 		}
-		goto restart;
 	    }
 	}
     }
-    if( !any )
-	return 0;
 
-  restart_sig:
-    /* now we may have duplicate signatures on one user ID: fix this */
-    for( in_uid = 0, n = *keyblock; n; n = n->next ) {
-	if( n->pkt->pkttype == PKT_USER_ID )
-	    in_uid = 1;
-	else if( n->pkt->pkttype == PKT_PUBLIC_SUBKEY
-		 || n->pkt->pkttype == PKT_SECRET_SUBKEY )
-	    in_uid = 0;
-	else if( in_uid ) {
-	    n2 = n;
-	    do {
-		KBNODE ncmp = NULL;
-		for( ; n2; n2 = n2->next ) {
-		    if(    n2->pkt->pkttype == PKT_USER_ID
-			|| n2->pkt->pkttype == PKT_PUBLIC_SUBKEY
-			|| n2->pkt->pkttype == PKT_SECRET_SUBKEY )
-			break;
-		    if( n2->pkt->pkttype != PKT_SIGNATURE )
-			;
-		    else if( !ncmp )
-			ncmp = n2;
-		    else if( !cmp_signatures( ncmp->pkt->pkt.signature,
-						n2->pkt->pkt.signature )) {
-			remove_kbnode( keyblock, n2 );
-			goto restart_sig;
-		    }
-		}
-		n2 = ncmp? ncmp->next : NULL;
-	    } while( n2 );
-	}
+  if(any && !opt.quiet)
+    {
+      const char *key="???";
+
+      if( (uid1=find_kbnode( *keyblock, PKT_PUBLIC_KEY )) )
+	key=keystr_from_pk(uid1->pkt->pkt.public_key);
+      else if( (uid1 = find_kbnode( *keyblock, PKT_SECRET_KEY )) )
+	key=keystr_from_sk(uid1->pkt->pkt.secret_key);
+
+      log_info(_("key %s: duplicated user ID detected - merged\n"),key);
     }
 
-    if(!opt.quiet)
-      {
-	const char *key="???";
-
-	if( (n = find_kbnode( *keyblock, PKT_PUBLIC_KEY )) )
-	  key=keystr_from_pk(n->pkt->pkt.public_key);
-	else if( (n = find_kbnode( *keyblock, PKT_SECRET_KEY )) )
-	  key=keystr_from_sk(n->pkt->pkt.secret_key);
-
-	log_info(_("key %s: duplicated user ID detected - merged\n"),key);
-      }
-
-    return 1;
+  return any;
 }
 
 /* Check for a 0x20 revocation from a revocation key that is not
@@ -2334,7 +2348,8 @@ auto_create_card_key_stub ( const char *serialnostr,
     size_t an;
 
     fingerprint_from_pk (pk, afp, &an);
-    memset (afp, 0, MAX_FINGERPRINT_LEN);
+    if (an < MAX_FINGERPRINT_LEN)
+      memset (afp+an, 0, MAX_FINGERPRINT_LEN-an);
     rc = keydb_search_fpr (hd, afp);
   }
 
