@@ -1,12 +1,12 @@
 /* parse-packet.c  - read packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -38,6 +36,11 @@
 #include "main.h"
 #include "i18n.h"
 
+#ifndef MAX_EXTERN_MPI_BITS
+#define MAX_EXTERN_MPI_BITS 16384
+#endif
+
+
 static int mpi_print_mode;
 static int list_mode;
 static FILE *listfp;
@@ -53,6 +56,7 @@ static int  copy_packet( IOBUF inp, IOBUF out, int pkttype,
 static void skip_packet( IOBUF inp, int pkttype,
 			 unsigned long pktlen, int partial );
 static void *read_rest( IOBUF inp, size_t pktlen, int partial );
+static int  parse_marker( IOBUF inp, int pkttype, unsigned long pktlen );
 static int  parse_symkeyenc( IOBUF inp, int pkttype, unsigned long pktlen,
 							     PACKET *packet );
 static int  parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen,
@@ -526,6 +530,9 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
       case PKT_GPG_CONTROL:
         rc = parse_gpg_control(inp, pkttype, pktlen, pkt, partial );
         break;
+    case PKT_MARKER:
+        rc = parse_marker(inp,pkttype,pktlen);
+	break;
       default:
 	skip_packet(inp, pkttype, pktlen, partial);
 	break;
@@ -590,32 +597,31 @@ copy_packet( IOBUF inp, IOBUF out, int pkttype,
 static void
 skip_packet( IOBUF inp, int pkttype, unsigned long pktlen, int partial )
 {
-    if( list_mode ) {
-	if( pkttype == PKT_MARKER )
-	    fputs(":marker packet:\n", listfp );
-	else
-	    fprintf (listfp, ":unknown packet: type %2d, length %lu\n",
-                     pkttype, pktlen);
-	if( pkttype ) {
-	    int c, i=0 ;
-	    if( pkttype != PKT_MARKER )
-		fputs("dump:", listfp );
-	    if( partial ) {
-		while( (c=iobuf_get(inp)) != -1 )
-		    dump_hex_line(c, &i);
+  if( list_mode )
+    {
+      fprintf (listfp, ":unknown packet: type %2d, length %lu\n",
+	       pkttype, pktlen);
+      if( pkttype )
+	{
+	  int c, i=0 ;
+	  fputs("dump:", listfp );
+	  if( partial )
+	    {
+	      while( (c=iobuf_get(inp)) != -1 )
+		dump_hex_line(c, &i);
 	    }
-	    else {
-		for( ; pktlen; pktlen-- )
-		    dump_hex_line(iobuf_get(inp), &i);
+	  else
+	    {
+	      for( ; pktlen; pktlen-- )
+		dump_hex_line(iobuf_get(inp), &i);
 	    }
-	    putc ('\n', listfp);
-	    return;
+	  putc ('\n', listfp);
+	  return;
 	}
     }
-    iobuf_skip_rest(inp,pktlen,partial);
+  iobuf_skip_rest(inp,pktlen,partial);
 }
 
-
 static void *
 read_rest( IOBUF inp, size_t pktlen, int partial )
 {
@@ -634,7 +640,40 @@ read_rest( IOBUF inp, size_t pktlen, int partial )
     return p;
 }
 
+static int
+parse_marker( IOBUF inp, int pkttype, unsigned long pktlen )
+{
+  if(pktlen!=3)
+    goto fail;
 
+  if(iobuf_get(inp)!='P')
+    {
+      pktlen--;
+      goto fail;
+    }
+
+  if(iobuf_get(inp)!='G')
+    {
+      pktlen--;
+      goto fail;
+    }
+
+  if(iobuf_get(inp)!='P')
+    {
+      pktlen--;
+      goto fail;
+    }
+
+  if(list_mode)
+    fputs(":marker packet: PGP\n", listfp );
+
+  return 0;
+
+ fail:
+  log_error("invalid marker packet\n");
+  iobuf_skip_rest(inp,pktlen,0);
+  return G10ERR_INVALID_PACKET;
+}
 
 static int
 parse_symkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
@@ -1419,7 +1458,7 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 
     if( list_mode ) {
 	fprintf (listfp, ":signature packet: algo %d, keyid %08lX%08lX\n"
-	       "\tversion %d, created %lu, md5len %d, sigclass %02x\n"
+	       "\tversion %d, created %lu, md5len %d, sigclass 0x%02x\n"
 	       "\tdigest algo %d, begin of digest %02x %02x\n",
 		sig->pubkey_algo,
 		(ulong)sig->keyid[0], (ulong)sig->keyid[1],
@@ -1437,10 +1476,21 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 	if( list_mode )
 	    fprintf (listfp, "\tunknown algorithm %d\n", sig->pubkey_algo );
 	unknown_pubkey_warning( sig->pubkey_algo );
-	/* we store the plain material in data[0], so that we are able
+	/* We store the plain material in data[0], so that we are able
 	 * to write it back with build_packet() */
-	sig->data[0]= mpi_set_opaque(NULL, read_rest(inp, pktlen, 0), pktlen );
-	pktlen = 0;
+        if (pktlen > (5 * MAX_EXTERN_MPI_BITS/8))
+          {
+            /* However we include a limit to avoid too trivial DoS
+               attacks by having gpg allocate too much memory.  */
+	    log_error ("signature packet: too much data\n");
+	    rc = G10ERR_INVALID_PACKET;
+          }
+        else
+          {
+            sig->data[0]= mpi_set_opaque (NULL, read_rest(inp, pktlen, 0),
+                                          pktlen );
+            pktlen = 0;
+          }
     }
     else {
 	for( i=0; i < ndata; i++ ) {
@@ -1489,7 +1539,7 @@ parse_onepass_sig( IOBUF inp, int pkttype, unsigned long pktlen,
     ops->last = iobuf_get_noeof(inp); pktlen--;
     if( list_mode )
 	fprintf (listfp, ":onepass_sig packet: keyid %08lX%08lX\n"
-	       "\tversion %d, sigclass %02x, digest %d, pubkey %d, last=%d\n",
+	      "\tversion %d, sigclass 0x%02x, digest %d, pubkey %d, last=%d\n",
 		(ulong)ops->keyid[0], (ulong)ops->keyid[1],
 		version, ops->sig_class,
 		ops->digest_algo, ops->pubkey_algo, ops->last );
@@ -1533,7 +1583,7 @@ read_protected_v3_mpi (IOBUF inp, unsigned long *length)
   buf = p = xmalloc (2 + nbytes);
   *p++ = nbits >> 8;
   *p++ = nbits;
-  for (; nbytes && length; nbytes--, --*length)
+  for (; nbytes && *length; nbytes--, --*length)
     *p++ = iobuf_get (inp);
   if (nbytes)
     {
