@@ -1,11 +1,12 @@
 /* gpgkeys_ldap.c - talk to a LDAP keyserver
- * Copyright (C) 2001, 2002, 2004, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +15,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * gives permission to link the code of the keyserver helper tools:
+ * gpgkeys_ldap, gpgkeys_curl and gpgkeys_hkp with the OpenSSL
+ * project's "OpenSSL" library (or with modified versions of it that
+ * use the same license as the "OpenSSL" library), and distribute the
+ * linked executables.  You must obey the GNU General Public License
+ * in all respects for all of the code used other than "OpenSSL".  If
+ * you modify this file, you may extend this exception to your version
+ * of the file, but you are not obligated to do so.  If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 
 #include <config.h>
@@ -29,6 +39,7 @@
 #endif
 #include <stdlib.h>
 #include <errno.h>
+#include <assert.h>
 
 #ifdef _WIN32
 #include <winsock2.h>
@@ -42,7 +53,12 @@
 #include <ldap.h>
 #endif
 
-#include "util.h"
+/* For mozldap */
+#ifdef HAVE_LDAP_SSL_H
+#include <ldap_ssl.h>
+#endif
+
+#include "compat.h"
 #include "keyserver.h"
 #include "ksutil.h"
 
@@ -119,7 +135,7 @@ key_in_keylist(const char *key,struct keylist *list)
 
   while(keyptr!=NULL)
     {
-      if(strcasecmp(key,keyptr->str)==0)
+      if(ascii_strcasecmp(key,keyptr->str)==0)
 	return 1;
 
       keyptr=keyptr->next;
@@ -258,7 +274,7 @@ make_one_attr(LDAPMod ***modlist,char *attr,const char *value)
   /* Search modlist for the attribute we're playing with. */
   for(m=*modlist;*m;m++)
     {
-      if(strcasecmp((*m)->mod_type,attr)==0)
+      if(ascii_strcasecmp((*m)->mod_type,attr)==0)
 	{
 	  char **ptr=(*m)->mod_values;
 	  int numvalues=0;
@@ -355,7 +371,7 @@ build_attrs(LDAPMod ***modlist,char *line)
   if((record=strsep(&line,":"))==NULL)
     return;
 
-  if(ascii_strcasecmp("pub",record)==0)
+  if (strcmp("pub",record)==0)
     {
       char *tok;
       int disabled=0,revoked=0;
@@ -461,7 +477,7 @@ build_attrs(LDAPMod ***modlist,char *line)
       make_one_attr(modlist,"pgpDisabled",disabled?"1":"0");
       make_one_attr(modlist,"pgpRevoked",revoked?"1":"0");
     }
-  else if(ascii_strcasecmp("sub",record)==0)
+  else if (strcmp("sub",record)==0)
     {
       char *tok;
 
@@ -499,7 +515,7 @@ build_attrs(LDAPMod ***modlist,char *line)
       /* Ignore the rest of the items for subkeys since the LDAP
 	 schema doesn't store them. */
     }
-  else if(ascii_strcasecmp("uid",record)==0)
+  else if (strcmp("uid",record)==0)
     {
       char *userid,*tok;
 
@@ -536,7 +552,7 @@ build_attrs(LDAPMod ***modlist,char *line)
 
       make_one_attr(modlist,"pgpUserID",userid);
     }
-  else if(ascii_strcasecmp("sig",record)==0)
+  else if(strcmp("sig",record)==0)
     {
       char *tok;
 
@@ -567,7 +583,7 @@ send_key(int *eof)
 {
   int err,begin=0,end=0,keysize=1,ret=KEYSERVER_INTERNAL_ERROR;
   char *dn=NULL,line[MAX_LINE],*key=NULL;
-  char keyid[17];
+  char keyid[17],state[6];
   LDAPMod **modlist,**addlist,**ml;
 
   modlist=malloc(sizeof(LDAPMod *));
@@ -608,7 +624,8 @@ send_key(int *eof)
   /* Assemble the INFO stuff into LDAP attributes */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"INFO %16s BEGIN\n",keyid)==1)
+    if(sscanf(line,"INFO%*[ ]%16s%*[ ]%5s\n",keyid,state)==2
+       && strcmp(state,"BEGIN")==0)
       {
 	begin=1;
 	break;
@@ -653,7 +670,8 @@ send_key(int *eof)
   /* Now parse each line until we see the END */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"INFO %16s END\n",keyid)==1)
+    if(sscanf(line,"INFO%*[ ]%16s%*[ ]%3s\n",keyid,state)==2
+       && strcmp(state,"END")==0)
       {
 	end=1;
 	break;
@@ -674,7 +692,8 @@ send_key(int *eof)
   /* Read and throw away stdin until we see the BEGIN */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"KEY %16s BEGIN\n",keyid)==1)
+    if(sscanf(line,"KEY%*[ ]%16s%*[ ]%5s\n",keyid,state)==2
+       && strcmp(state,"BEGIN")==0)
       {
 	begin=1;
 	break;
@@ -692,7 +711,8 @@ send_key(int *eof)
   /* Now slurp up everything until we see the END */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"KEY %16s END\n",keyid)==1)
+    if(sscanf(line,"KEY%*[ ]%16s%*[ ]%3s\n",keyid,state)==2
+       && strcmp(state,"END")==0)
       {
 	end=1;
 	break;
@@ -766,6 +786,7 @@ send_key(int *eof)
   free(modlist);
   free(addlist);
   free(dn);
+  free(key);
 
   if(ret!=0 && begin)
     fprintf(output,"KEY %s FAILED %d\n",keyid,ret);
@@ -778,7 +799,7 @@ send_key_keyserver(int *eof)
 {
   int err,begin=0,end=0,keysize=1,ret=KEYSERVER_INTERNAL_ERROR;
   char *dn=NULL,line[MAX_LINE],*key[2]={NULL,NULL};
-  char keyid[17];
+  char keyid[17],state[6];
   LDAPMod mod, *attrs[2];
 
   memset(&mod,0,sizeof(mod));
@@ -812,7 +833,8 @@ send_key_keyserver(int *eof)
   /* Read and throw away stdin until we see the BEGIN */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"KEY %16s BEGIN\n",keyid)==1)
+    if(sscanf(line,"KEY%*[ ]%16s%*[ ]%5s\n",keyid,state)==2
+       && strcmp(state,"BEGIN")==0)
       {
 	begin=1;
 	break;
@@ -830,7 +852,8 @@ send_key_keyserver(int *eof)
   /* Now slurp up everything until we see the END */
 
   while(fgets(line,MAX_LINE,input)!=NULL)
-    if(sscanf(line,"KEY %16s END\n",keyid)==1)
+    if(sscanf(line,"KEY%*[ ]%16s%*[ ]%3s\n",keyid,state)==2
+       && strcmp(state,"END")==0)
       {
 	end=1;
 	break;
@@ -1121,36 +1144,25 @@ get_key(char *getkey)
 
 #define LDAP_ESCAPE_CHARS "*()\\"
 
-static int
+/* Append string to buffer in a LDAP-quoted way */
+static void
 ldap_quote(char *buffer,const char *string)
 {
-  int count=0;
+  /* Find the end of buffer */
+  buffer+=strlen(buffer);
 
   for(;*string;string++)
     {
       if(strchr(LDAP_ESCAPE_CHARS,*string))
 	{
-	  if(buffer)
-	    {
-	      sprintf(buffer,"\\%02X",*string);
-	      buffer+=3;
-	    }
-
-	  count+=3;
+	  sprintf(buffer,"\\%02X",*string);
+	  buffer+=3;
 	}
       else
-	{
-	  if(buffer)
-	    *buffer++=*string;
-
-	  count++;
-	}
+	*buffer++=*string;
     }
 
-  if(buffer)
-    *buffer='\0';
-
-  return count;
+  *buffer='\0';
 }
 
 /* Note that key-not-found is not a fatal error */
@@ -1159,10 +1171,9 @@ get_name(char *getkey)
 {
   LDAPMessage *res,*each;
   int ret=KEYSERVER_INTERNAL_ERROR,err,count;
-  char *expanded_search;
   /* The maximum size of the search, including the optional stuff and
      the trailing \0 */
-  char search[2+11+3+MAX_LINE+2+15+14+1+1+20];
+  char search[2+12+(MAX_LINE*3)+2+15+14+1+1+20];
   /* This ordering is significant - specifically, "pgpcertid" needs to
      be the second item in the list, since everything after it may be
      discarded if the user isn't in verbose mode. */
@@ -1172,26 +1183,25 @@ get_name(char *getkey)
   attrs[0]=pgpkeystr; /* Some compilers don't like using variables as
                          array initializers. */
 
-  expanded_search=malloc(ldap_quote(NULL,getkey)+1);
-  if(!expanded_search)
-    {
-      fprintf(output,"NAME %s FAILED %d\n",getkey,KEYSERVER_NO_MEMORY);
-      fprintf(console,"Out of memory when quoting LDAP search string\n");
-      return KEYSERVER_NO_MEMORY;
-    }
-
-  ldap_quote(expanded_search,getkey);
-
   /* Build the search string */
 
-  sprintf(search,"%s(pgpuserid=*%s*)%s%s%s",
-	  (!(opt->flags.include_disabled&&opt->flags.include_revoked))?"(&":"",
-	  expanded_search,
-	  opt->flags.include_disabled?"":"(pgpdisabled=0)",
-	  opt->flags.include_revoked?"":"(pgprevoked=0)",
-	  !(opt->flags.include_disabled&&opt->flags.include_revoked)?")":"");
+  search[0]='\0';
 
-  free(expanded_search);
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,"(&");
+
+  strcat(search,"(pgpUserID=*");
+  ldap_quote(search,getkey);
+  strcat(search,"*)");
+
+  if(!opt->flags.include_disabled)
+    strcat(search,"(pgpDisabled=0)");
+
+  if(!opt->flags.include_revoked)
+    strcat(search,"(pgpRevoked=0)");
+
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,")");
 
   if(opt->verbose>2)
     fprintf(console,"gpgkeys: LDAP fetch for: %s\n",search);
@@ -1271,7 +1281,7 @@ printquoted(FILE *stream,char *string,char delim)
   while(*string)
     {
       if(*string==delim || *string=='%')
-	fprintf(stream,"%%%02x",*string);
+	fprintf(stream,"%%%02x",(unsigned char)*string);
       else
 	fputc(*string,stream);
 
@@ -1284,19 +1294,25 @@ printquoted(FILE *stream,char *string,char delim)
 static int
 search_key(const char *searchkey)
 {
-  char **vals;
+  char **vals,*search;
   LDAPMessage *res,*each;
   int err,count=0;
   struct keylist *dupelist=NULL;
-  char *expanded_search;
   /* The maximum size of the search, including the optional stuff and
      the trailing \0 */
-  char search[2+11+3+MAX_LINE+2+15+14+1+1+20];
   char *attrs[]={"pgpcertid","pgpuserid","pgprevoked","pgpdisabled",
 		 "pgpkeycreatetime","pgpkeyexpiretime","modifytimestamp",
 		 "pgpkeysize","pgpkeytype",NULL};
   enum ks_search_type search_type;
 
+  search=malloc(2+1+9+1+3+strlen(searchkey)+3+1+15+14+1+1+20);
+  if(!search)
+    {
+      fprintf(console,"gpgkeys: out of memory when building search list\n");
+      fprintf(output,"SEARCH %s FAILED %d\n",searchkey,KEYSERVER_NO_MEMORY);
+      return KEYSERVER_NO_MEMORY;
+    }
+
   fprintf(output,"SEARCH %s BEGIN\n",searchkey);
 
   search_type=classify_ks_search(&searchkey);
@@ -1305,37 +1321,91 @@ search_key(const char *searchkey)
     fprintf(console,"search type is %d, and key is \"%s\"\n",
 	    search_type,searchkey);
 
-  expanded_search=malloc(ldap_quote(NULL,searchkey)+1);
-  if(!expanded_search)
-    {
-      fprintf(output,"SEARCH %s FAILED %d\n",searchkey,KEYSERVER_NO_MEMORY);
-      fprintf(console,"Out of memory when quoting LDAP search string\n");
-      return KEYSERVER_NO_MEMORY;
-    }
-
-  ldap_quote(expanded_search,searchkey);
-
   /* Build the search string */
 
-  sprintf(search,"%s(pgpuserid=%s%s%s)%s%s%s",
-	  (!(opt->flags.include_disabled&&opt->flags.include_revoked))?"(&":"",
-	  (search_type==KS_SEARCH_EXACT)?"":
-	  (search_type==KS_SEARCH_MAILSUB)?"*<*":"*",
-	  expanded_search,
-	  (search_type==KS_SEARCH_EXACT
-	   || search_type==KS_SEARCH_MAIL)?"":
-	  (search_type==KS_SEARCH_MAILSUB)?"*>":"*",
-	  opt->flags.include_disabled?"":"(pgpdisabled=0)",
-	  opt->flags.include_revoked?"":"(pgprevoked=0)",
-	  !(opt->flags.include_disabled&&opt->flags.include_revoked)?")":"");
+  search[0]='\0';
 
-  free(expanded_search);
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,"(&");
+
+  strcat(search,"(");
+
+  switch(search_type)
+    {
+    case KS_SEARCH_KEYID_SHORT:
+      strcat(search,"pgpKeyID");
+      break;
+
+    case KS_SEARCH_KEYID_LONG:
+      strcat(search,"pgpCertID");
+      break;
+
+    default:
+      strcat(search,"pgpUserID");
+      break;
+    }
+
+  strcat(search,"=");
+
+  switch(search_type)
+    {
+    case KS_SEARCH_SUBSTR:
+      strcat(search,"*");
+      break;
+
+    case KS_SEARCH_MAIL:
+      strcat(search,"*<");
+      break;
+
+    case KS_SEARCH_MAILSUB:
+      strcat(search,"*<*");
+      break;
+
+    case KS_SEARCH_EXACT:
+    case KS_SEARCH_KEYID_LONG:
+    case KS_SEARCH_KEYID_SHORT:
+      break;
+    }
+
+  strcat(search,searchkey);
+
+  switch(search_type)
+    {
+    case KS_SEARCH_SUBSTR:
+      strcat(search,"*");
+      break;
+
+    case KS_SEARCH_MAIL:
+      strcat(search,">*");
+      break;
+
+    case KS_SEARCH_MAILSUB:
+      strcat(search,"*>*");
+      break;
+
+    case KS_SEARCH_EXACT:
+    case KS_SEARCH_KEYID_LONG:
+    case KS_SEARCH_KEYID_SHORT:
+      break;
+    }
+
+  strcat(search,")");
+
+  if(!opt->flags.include_disabled)
+    strcat(search,"(pgpDisabled=0)");
+
+  if(!opt->flags.include_revoked)
+    strcat(search,"(pgpRevoked=0)");
+
+  if(!opt->flags.include_disabled || !opt->flags.include_revoked)
+    strcat(search,")");
 
   if(opt->verbose>2)
     fprintf(console,"gpgkeys: LDAP search for: %s\n",search);
 
   err=ldap_search_s(ldap,basekeyspacedn,
 		    LDAP_SCOPE_SUBTREE,search,attrs,0,&res);
+  free(search);
   if(err!=LDAP_SUCCESS && err!=LDAP_SIZELIMIT_EXCEEDED)
     {
       int errtag=ldap_err_to_gpg_err(err);
@@ -1420,9 +1490,9 @@ search_key(const char *searchkey)
 		    {
 		      /* The LDAP server doesn't exactly handle this
 			 well. */
-		      if(strcasecmp(vals[0],"RSA")==0)
+		      if(ascii_strcasecmp(vals[0],"RSA")==0)
 			fprintf(output,"1");
-		      else if(strcasecmp(vals[0],"DSS/DH")==0)
+		      else if(ascii_strcasecmp(vals[0],"DSS/DH")==0)
 			fprintf(output,"17");
 		      ldap_value_free(vals);
 		    }
@@ -1503,7 +1573,7 @@ search_key(const char *searchkey)
 		      vals=ldap_get_values(ldap,uids,"pgpcertid");
 		      if(vals!=NULL)
 			{
-			  if(strcasecmp(certid[0],vals[0])==0)
+			  if(ascii_strcasecmp(certid[0],vals[0])==0)
 			    {
 			      char **uidvals;
 
@@ -1805,25 +1875,25 @@ main(int argc,char *argv[])
 
 	  optionstr[MAX_OPTION]='\0';
 
-	  if(strncasecmp(optionstr,"no-",3)==0)
+	  if(ascii_strncasecmp(optionstr,"no-",3)==0)
 	    {
 	      no=1;
 	      start=&optionstr[3];
 	    }
 
-	  if(strncasecmp(start,"tls",3)==0)
+	  if(ascii_strncasecmp(start,"tls",3)==0)
 	    {
 	      if(no)
 		use_tls=0;
 	      else if(start[3]=='=')
 		{
-		  if(strcasecmp(&start[4],"no")==0)
+		  if(ascii_strcasecmp(&start[4],"no")==0)
 		    use_tls=0;
-		  else if(strcasecmp(&start[4],"try")==0)
+		  else if(ascii_strcasecmp(&start[4],"try")==0)
 		    use_tls=1;
-		  else if(strcasecmp(&start[4],"warn")==0)
+		  else if(ascii_strcasecmp(&start[4],"warn")==0)
 		    use_tls=2;
-		  else if(strcasecmp(&start[4],"require")==0)
+		  else if(ascii_strcasecmp(&start[4],"require")==0)
 		    use_tls=3;
 		  else
 		    use_tls=1;
@@ -1831,7 +1901,7 @@ main(int argc,char *argv[])
 	      else if(start[3]=='\0')
 		use_tls=1;
 	    }
-	  else if(strncasecmp(start,"basedn",6)==0)
+	  else if(ascii_strncasecmp(start,"basedn",6)==0)
 	    {
 	      if(no)
 		{
@@ -1853,7 +1923,7 @@ main(int argc,char *argv[])
 		  real_ldap=1;
 		}
 	    }
-	  else if(strncasecmp(start,"binddn",6)==0)
+	  else if(ascii_strncasecmp(start,"binddn",6)==0)
 	    {
 	      if(no)
 		{
@@ -1875,7 +1945,7 @@ main(int argc,char *argv[])
 		  real_ldap=1;
 		}
 	    }
-	  else if(strncasecmp(start,"bindpw",6)==0)
+	  else if(ascii_strncasecmp(start,"bindpw",6)==0)
 	    {
 	      if(no)
 		{
@@ -1909,7 +1979,7 @@ main(int argc,char *argv[])
       goto fail;
     }
 
-  if(strcasecmp(opt->scheme,"ldaps")==0)
+  if(ascii_strcasecmp(opt->scheme,"ldaps")==0)
     {
       port=636;
       use_ssl=1;
@@ -2116,7 +2186,7 @@ main(int argc,char *argv[])
 	      else
 		ver=LDAP_OPT_X_TLS_NEVER;
 
-	      err=ldap_set_option(ldap,LDAP_OPT_X_TLS_REQUIRE_CERT,&ver);
+	      err=ldap_set_option(NULL,LDAP_OPT_X_TLS_REQUIRE_CERT,&ver);
 	    }
 #endif
 
@@ -2252,7 +2322,7 @@ main(int argc,char *argv[])
 	  keyptr=keyptr->next;
 	}
 
-      searchkey=malloc(len+1);
+      searchkey=malloc((len*3)+1);
       if(searchkey==NULL)
 	{
 	  ret=KEYSERVER_NO_MEMORY;
@@ -2265,7 +2335,7 @@ main(int argc,char *argv[])
       keyptr=keylist;
       while(keyptr!=NULL)
 	{
-	  strcat(searchkey,keyptr->str);
+	  ldap_quote(searchkey,keyptr->str);
 	  strcat(searchkey,"*");
 	  keyptr=keyptr->next;
 	}
@@ -2280,7 +2350,7 @@ main(int argc,char *argv[])
       free(searchkey);
     }
   else
-    BUG();
+    assert (!"invalid action");
 
   if(!failed)
     ret=KEYSERVER_OK;
